@@ -28,6 +28,7 @@ import six
 import webob
 from werkzeug import exceptions
 
+from octavia.amphorae.backends.agent.api_server import haproxy_compatibility
 from octavia.amphorae.backends.agent.api_server import osutils
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.amphorae.backends.utils import haproxy_query as query
@@ -42,6 +43,7 @@ CONF = cfg.CONF
 UPSTART_CONF = 'upstart.conf.j2'
 SYSVINIT_CONF = 'sysvinit.conf.j2'
 SYSTEMD_CONF = 'systemd.conf.j2'
+AMPHORA_NETNS = 'amphora-netns'
 
 JINJA_ENV = jinja2.Environment(
     autoescape=True,
@@ -124,6 +126,10 @@ class Listener(object):
         # result an error when haproxy starts.
         new_config = re.sub(r"\s+group\s.+", "", s_io.getvalue())
 
+        # Handle any haproxy version compatibility issues
+        new_config = haproxy_compatibility.process_cfg_for_version_compat(
+            new_config)
+
         with os.fdopen(os.open(name, flags, mode), 'w') as file:
             file.write(new_config)
 
@@ -135,7 +141,7 @@ class Listener(object):
         try:
             subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
         except subprocess.CalledProcessError as e:
-            LOG.error("Failed to verify haproxy file: %s", e)
+            LOG.error("Failed to verify haproxy file: %s %s", e, e.output)
             # Save the last config that failed validation for debugging
             os.rename(name, ''.join([name, '-failed']))
             return webob.Response(
@@ -180,6 +186,22 @@ class Listener(object):
             # mode 00755
             mode = (stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
                     stat.S_IROTH | stat.S_IXOTH)
+
+        if init_system == consts.INIT_SYSTEMD:
+            # TODO(bcafarel): implement this for other init systems
+            # netns handling depends on a separate unit file
+            netns_path = os.path.join(consts.SYSTEMD_DIR,
+                                      AMPHORA_NETNS + '.service')
+            if not os.path.exists(netns_path):
+                with os.fdopen(os.open(netns_path, flags, mode),
+                               'w') as text_file:
+                    text = JINJA_ENV.get_template(AMPHORA_NETNS +
+                                                  '.systemd.j2').render(
+                            amphora_nsname=consts.AMPHORA_NAMESPACE,
+                            HasIFUPAll=self._osutils.has_ifup_all()
+                            )
+                    text_file.write(text)
+
         if not os.path.exists(init_path):
             with os.fdopen(os.open(init_path, flags, mode), 'w') as text_file:
 
@@ -192,6 +214,7 @@ class Listener(object):
                     respawn_count=util.CONF.haproxy_amphora.respawn_count,
                     respawn_interval=(util.CONF.haproxy_amphora.
                                       respawn_interval),
+                    amphora_netns=AMPHORA_NETNS,
                     amphora_nsname=consts.AMPHORA_NAMESPACE,
                     HasIFUPAll=self._osutils.has_ifup_all()
                 )
@@ -203,8 +226,9 @@ class Listener(object):
                 subprocess.check_output(init_enable_cmd.split(),
                                         stderr=subprocess.STDOUT)
             except subprocess.CalledProcessError as e:
-                LOG.error("Failed to enable haproxy-%(list)s service: %(err)s",
-                          {'list': listener_id, 'err': e})
+                LOG.error("Failed to enable haproxy-%(list)s service: "
+                          "%(err)s %(out)s", {'list': listener_id, 'err': e,
+                                              'out': e.output})
                 return webob.Response(json=dict(
                     message="Error enabling haproxy-{0} service".format(
                             listener_id), details=e.output), status=500)
@@ -244,8 +268,10 @@ class Listener(object):
             subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
         except subprocess.CalledProcessError as e:
             if 'Job is already running' not in e.output:
-                LOG.debug("Failed to %(action)s HAProxy service: %(err)s",
-                          {'action': action, 'err': e})
+                LOG.debug(
+                    "Failed to %(action)s haproxy-%(list)s service: %(err)s "
+                    "%(out)s", {'action': action, 'list': listener_id,
+                                'err': e, 'out': e.output})
                 return webob.Response(json=dict(
                     message="Error {0}ing haproxy".format(action),
                     details=e.output), status=500)
@@ -277,7 +303,8 @@ class Listener(object):
             try:
                 subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
             except subprocess.CalledProcessError as e:
-                LOG.error("Failed to stop HAProxy service: %s", e)
+                LOG.error("Failed to stop haproxy-%s service: %s %s",
+                          listener_id, e, e.output)
                 return webob.Response(json=dict(
                     message="Error stopping haproxy",
                     details=e.output), status=500)
@@ -320,7 +347,8 @@ class Listener(object):
                                         stderr=subprocess.STDOUT)
             except subprocess.CalledProcessError as e:
                 LOG.error("Failed to disable haproxy-%(list)s service: "
-                          "%(err)s", {'list': listener_id, 'err': e})
+                          "%(err)s %(out)s", {'list': listener_id, 'err': e,
+                                              'out': e.output})
                 return webob.Response(json=dict(
                     message="Error disabling haproxy-{0} service".format(
                             listener_id), details=e.output), status=500)
@@ -408,7 +436,7 @@ class Listener(object):
         mode = stat.S_IRUSR | stat.S_IWUSR
         with os.fdopen(os.open(file, flags, mode), 'wb') as crt_file:
             b = stream.read(BUFFER)
-            while (b):
+            while b:
                 crt_file.write(b)
                 b = stream.read(BUFFER)
 
