@@ -100,6 +100,7 @@ class BaseOS(object):
                 broadcast=broadcast,
                 netmask=netmask,
                 gateway=gateway,
+                network=utils.ip_netmask_to_cidr(vip, netmask),
                 mtu=mtu,
                 vrrp_ip=vrrp_ip,
                 vrrp_ipv6=vrrp_version is 6,
@@ -222,7 +223,7 @@ class Ubuntu(BaseOS):
         return os_name in ['Ubuntu']
 
     def cmd_get_version_of_installed_package(self, package_name):
-        return "dpkg --status {name}".format(name=package_name)
+        return "dpkg-query -W -f=${{Version}} {name}".format(name=package_name)
 
     def get_network_interface_file(self, interface):
         if CONF.amphora_agent.agent_server_network_file:
@@ -306,7 +307,8 @@ class RH(BaseOS):
         return os_name in ['fedora', 'redhat', 'centos']
 
     def cmd_get_version_of_installed_package(self, package_name):
-        return "rpm -qi {name}".format(name=package_name)
+        return "rpm -q --queryformat %{{VERSION}} {name}".format(
+            name=package_name)
 
     def get_network_interface_file(self, interface):
         if CONF.amphora_agent.agent_server_network_file:
@@ -385,7 +387,7 @@ class RH(BaseOS):
 
         self.write_static_routes_interface_file(
             routes_interface_file_path, primary_interface,
-            render_host_routes, template_routes, gateway, vip)
+            render_host_routes, template_routes, gateway, vip, netmask)
 
         route_rules_interface_file_path = (
             self.get_route_rules_interface_file(primary_interface))
@@ -393,11 +395,12 @@ class RH(BaseOS):
 
         self.write_static_routes_interface_file(
             route_rules_interface_file_path, primary_interface,
-            render_host_routes, template_rules, gateway, vip)
+            render_host_routes, template_rules, gateway, vip, netmask)
 
     def write_static_routes_interface_file(self, interface_file_path,
                                            interface, host_routes,
-                                           template_routes, gateway, vip):
+                                           template_routes, gateway,
+                                           vip, netmask):
         # write static routes interface file
 
         mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
@@ -414,6 +417,7 @@ class RH(BaseOS):
                 interface=interface,
                 host_routes=host_routes,
                 gateway=gateway,
+                network=utils.ip_netmask_to_cidr(vip, netmask),
                 vip=vip,
             )
             text_file.write(text)
@@ -441,7 +445,7 @@ class RH(BaseOS):
 
             self.write_static_routes_interface_file(
                 routes_interface_file_path, netns_interface,
-                host_routes, template_routes, None, None)
+                host_routes, template_routes, None, None, None)
 
     def bring_interfaces_up(self, ip, primary_interface, secondary_interface):
         if ip.version == 4:
