@@ -43,8 +43,9 @@ class L7PolicyController(base.BaseController):
         super(L7PolicyController, self).__init__()
         self.handler = self.handler.l7policy
 
-    @wsme_pecan.wsexpose(l7policy_types.L7PolicyRootResponse, wtypes.text)
-    def get(self, id):
+    @wsme_pecan.wsexpose(l7policy_types.L7PolicyRootResponse, wtypes.text,
+                         [wtypes.text], ignore_extra_args=True)
+    def get(self, id, fields=None):
         """Gets a single l7policy's details."""
         context = pecan.request.context.get('octavia_context')
         db_l7policy = self._get_db_l7policy(context.session, id)
@@ -52,8 +53,10 @@ class L7PolicyController(base.BaseController):
         self._auth_validate_action(context, db_l7policy.project_id,
                                    constants.RBAC_GET_ONE)
 
-        result = self._convert_db_to_type(db_l7policy,
-                                          l7policy_types.L7PolicyResponse)
+        result = self._convert_db_to_type(
+            db_l7policy, l7policy_types.L7PolicyResponse)
+        if fields is not None:
+            result = self._filter_fields([result], fields)[0]
         return l7policy_types.L7PolicyRootResponse(l7policy=result)
 
     @wsme_pecan.wsexpose(l7policy_types.L7PoliciesRootResponse, wtypes.text,
@@ -107,9 +110,8 @@ class L7PolicyController(base.BaseController):
         try:
             return self.repositories.l7policy.create(lock_session,
                                                      **l7policy_dict)
-        except odb_exceptions.DBDuplicateEntry as de:
-            if ['id'] == de.columns:
-                raise exceptions.IDAlreadyExists()
+        except odb_exceptions.DBDuplicateEntry:
+            raise exceptions.IDAlreadyExists()
         except odb_exceptions.DBError:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
@@ -157,18 +159,18 @@ class L7PolicyController(base.BaseController):
                                    constants.RBAC_POST)
 
         lock_session = db_api.get_session(autocommit=False)
-        if self.repositories.check_quota_met(
-                context.session,
-                lock_session,
-                data_models.L7Policy,
-                l7policy.project_id):
-            lock_session.rollback()
-            raise exceptions.QuotaException
-
-        l7policy_dict = db_prepare.create_l7policy(
-            l7policy.to_dict(render_unsets=True),
-            load_balancer_id, listener_id)
         try:
+            if self.repositories.check_quota_met(
+                    context.session,
+                    lock_session,
+                    data_models.L7Policy,
+                    l7policy.project_id):
+                raise exceptions.QuotaException
+
+            l7policy_dict = db_prepare.create_l7policy(
+                l7policy.to_dict(render_unsets=True),
+                load_balancer_id, listener_id)
+
             self._test_lb_and_listener_statuses(
                 lock_session, lb_id=load_balancer_id,
                 listener_ids=[listener_id])
@@ -261,6 +263,9 @@ class L7PolicyController(base.BaseController):
         self._auth_validate_action(context, db_l7policy.project_id,
                                    constants.RBAC_DELETE)
 
+        if db_l7policy.provisioning_status == constants.DELETED:
+            return
+
         self._test_lb_and_listener_statuses(context.session,
                                             lb_id=load_balancer_id,
                                             listener_ids=[listener_id])
