@@ -41,8 +41,9 @@ class L7RuleController(base.BaseController):
         self.l7policy_id = l7policy_id
         self.handler = self.handler.l7rule
 
-    @wsme_pecan.wsexpose(l7rule_types.L7RuleRootResponse, wtypes.text)
-    def get(self, id):
+    @wsme_pecan.wsexpose(l7rule_types.L7RuleRootResponse, wtypes.text,
+                         [wtypes.text], ignore_extra_args=True)
+    def get(self, id, fields=None):
         """Gets a single l7rule's details."""
         context = pecan.request.context.get('octavia_context')
         db_l7rule = self._get_db_l7rule(context.session, id)
@@ -50,12 +51,14 @@ class L7RuleController(base.BaseController):
         self._auth_validate_action(context, db_l7rule.project_id,
                                    constants.RBAC_GET_ONE)
 
-        result = self._convert_db_to_type(db_l7rule,
-                                          l7rule_types.L7RuleResponse)
+        result = self._convert_db_to_type(
+            db_l7rule, l7rule_types.L7RuleResponse)
+        if fields is not None:
+            result = self._filter_fields([result], fields)[0]
         return l7rule_types.L7RuleRootResponse(rule=result)
 
-    @wsme_pecan.wsexpose(l7rule_types.L7RulesRootResponse, wtypes.text,
-                         [wtypes.text], ignore_extra_args=True)
+    @wsme_pecan.wsexpose(l7rule_types.L7RulesRootResponse, [wtypes.text],
+                         ignore_extra_args=True)
     def get_all(self, fields=None):
         """Lists all l7rules of a l7policy."""
         pcontext = pecan.request.context
@@ -116,10 +119,9 @@ class L7RuleController(base.BaseController):
     def _validate_create_l7rule(self, lock_session, l7rule_dict):
         try:
             return self.repositories.l7rule.create(lock_session, **l7rule_dict)
-        except odb_exceptions.DBDuplicateEntry as de:
-            if ['id'] == de.columns:
-                raise exceptions.IDAlreadyExists()
-        except odb_exceptions.DBError as de:
+        except odb_exceptions.DBDuplicateEntry:
+            raise exceptions.IDAlreadyExists()
+        except odb_exceptions.DBError:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
             # do not give any information as to what constraint failed
@@ -160,9 +162,10 @@ class L7RuleController(base.BaseController):
                                    constants.RBAC_POST)
 
         lock_session = db_api.get_session(autocommit=False)
-        l7rule_dict = db_prepare.create_l7rule(
-            l7rule.to_dict(render_unsets=True), self.l7policy_id)
         try:
+            l7rule_dict = db_prepare.create_l7rule(
+                l7rule.to_dict(render_unsets=True), self.l7policy_id)
+
             self._test_lb_listener_policy_statuses(context.session)
 
             db_l7rule = self._validate_create_l7rule(lock_session, l7rule_dict)
@@ -233,6 +236,9 @@ class L7RuleController(base.BaseController):
         self._auth_validate_action(context, db_l7rule.project_id,
                                    constants.RBAC_DELETE)
 
+        if db_l7rule.provisioning_status == constants.DELETED:
+            return
+
         self._test_lb_listener_policy_statuses(context.session)
 
         self.repositories.l7rule.update(
