@@ -63,8 +63,9 @@ class ListenersController(base.BaseController):
                 resource=data_models.Listener._name(), id=id)
         return db_listener
 
-    @wsme_pecan.wsexpose(listener_types.ListenerRootResponse, wtypes.text)
-    def get_one(self, id):
+    @wsme_pecan.wsexpose(listener_types.ListenerRootResponse, wtypes.text,
+                         [wtypes.text], ignore_extra_args=True)
+    def get_one(self, id, fields=None):
         """Gets a single listener's details."""
         context = pecan.request.context.get('octavia_context')
         db_listener = self._get_db_listener(context.session, id)
@@ -74,6 +75,8 @@ class ListenersController(base.BaseController):
 
         result = self._convert_db_to_type(db_listener,
                                           listener_types.ListenerResponse)
+        if fields is not None:
+            result = self._filter_fields([result], fields)[0]
         return listener_types.ListenerRootResponse(listener=result)
 
     @wsme_pecan.wsexpose(listener_types.ListenersRootResponse, wtypes.text,
