@@ -35,6 +35,7 @@ import octavia.common.validate as validate
 from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
 from octavia.i18n import _
+from octavia.network import base as network_base
 
 
 CONF = cfg.CONF
@@ -48,8 +49,9 @@ class LoadBalancersController(base.BaseController):
         super(LoadBalancersController, self).__init__()
         self.handler = self.handler.load_balancer
 
-    @wsme_pecan.wsexpose(lb_types.LoadBalancerRootResponse, wtypes.text)
-    def get_one(self, id):
+    @wsme_pecan.wsexpose(lb_types.LoadBalancerRootResponse, wtypes.text,
+                         [wtypes.text], ignore_extra_args=True)
+    def get_one(self, id, fields=None):
         """Gets a single load balancer's details."""
         context = pecan.request.context.get('octavia_context')
         load_balancer = self._get_db_lb(context.session, id)
@@ -59,6 +61,8 @@ class LoadBalancersController(base.BaseController):
 
         result = self._convert_db_to_type(
             load_balancer, lb_types.LoadBalancerResponse)
+        if fields is not None:
+            result = self._filter_fields([result], fields)[0]
         return lb_types.LoadBalancerRootResponse(loadbalancer=result)
 
     @wsme_pecan.wsexpose(lb_types.LoadBalancersRootResponse, wtypes.text,
@@ -130,8 +134,18 @@ class LoadBalancersController(base.BaseController):
     @staticmethod
     def _validate_port_and_fill_or_validate_subnet(load_balancer):
         port = validate.port_exists(port_id=load_balancer.vip_port_id)
+        validate.check_port_in_use(port)
         load_balancer.vip_network_id = port.network_id
 
+        # validate the request vip port whether applied the qos_policy and
+        # store the port_qos_policy to loadbalancer obj if possible. The
+        # default behavior is that if 'vip_qos_policy_id' is specified in the
+        # request, it will override the qos_policy applied on vip_port.
+        port_qos_policy_id = port.qos_policy_id
+        if (port_qos_policy_id and
+                isinstance(load_balancer.vip_qos_policy_id, wtypes.UnsetType)):
+            load_balancer.vip_qos_policy_id = port_qos_policy_id
+
         # Identify the subnet for this port
         if load_balancer.vip_subnet_id:
             validate.subnet_exists(subnet_id=load_balancer.vip_subnet_id)
@@ -193,23 +207,25 @@ class LoadBalancersController(base.BaseController):
             subnet = validate.subnet_exists(
                 subnet_id=load_balancer.vip_subnet_id)
             load_balancer.vip_network_id = subnet.network_id
-
+        if load_balancer.vip_qos_policy_id:
+            validate.qos_policy_exists(
+                qos_policy_id=load_balancer.vip_qos_policy_id)
         validate.network_allowed_by_config(load_balancer.vip_network_id)
 
     def _create_vip_port_if_not_exist(self, load_balancer_db):
         """Create vip port."""
         network_driver = utils.get_network_driver()
         try:
-            vip = network_driver.allocate_vip(load_balancer_db)
-        except Exception as e:
+            return network_driver.allocate_vip(load_balancer_db)
+        except network_base.AllocateVIPException as e:
             # Convert neutron style exception to octavia style
             # if the error was API ready
-            if e.orig_code is not None:
+            if getattr(e, 'orig_code', None) is not None:
                 e.code = e.orig_code
+            if getattr(e, 'orig_msg', None) is not None:
                 e.message = e.orig_msg
                 e.msg = e.orig_msg
             raise e
-        return vip
 
     @wsme_pecan.wsexpose(lb_types.LoadBalancerFullRootResponse,
                          body=lb_types.LoadBalancerRootPOST, status_code=201)
@@ -231,16 +247,16 @@ class LoadBalancersController(base.BaseController):
         self._validate_vip_request_object(load_balancer)
 
         lock_session = db_api.get_session(autocommit=False)
-        if self.repositories.check_quota_met(
-                context.session,
-                lock_session,
-                data_models.LoadBalancer,
-                load_balancer.project_id):
-            lock_session.rollback()
-            raise exceptions.QuotaException
-
-        db_lb, db_pools, db_lists = None, None, None
         try:
+            if self.repositories.check_quota_met(
+                    context.session,
+                    lock_session,
+                    data_models.LoadBalancer,
+                    load_balancer.project_id):
+                raise exceptions.QuotaException
+
+            db_lb, db_pools, db_lists = None, None, None
+
             lb_dict = db_prepare.create_load_balancer(load_balancer.to_dict(
                 render_unsets=False
             ))
@@ -418,6 +434,11 @@ class LoadBalancersController(base.BaseController):
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_PUT)
 
+        if (load_balancer.vip_qos_policy_id and
+                not isinstance(load_balancer.vip_qos_policy_id,
+                               wtypes.UnsetType) and
+                db_lb.vip.qos_policy_id != load_balancer.vip_qos_policy_id):
+            validate.qos_policy_exists(load_balancer.vip_qos_policy_id)
         self._test_lb_status(context.session, id)
         try:
             LOG.info("Sending updated Load Balancer %s to the handler", id)
@@ -440,6 +461,9 @@ class LoadBalancersController(base.BaseController):
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_DELETE)
 
+        if db_lb.provisioning_status == constants.DELETED:
+            return
+
         with db_api.get_lock_session() as lock_session:
             if (db_lb.listeners or db_lb.pools) and not cascade:
                 msg = _("Cannot delete Load Balancer %s - "
@@ -462,17 +486,24 @@ class LoadBalancersController(base.BaseController):
     def _lookup(self, id, *remainder):
         """Overridden pecan _lookup method for custom routing.
 
-        Currently it checks if this was a statuses request and routes
-        the request to the StatusesController.
+        Currently it checks if this was a status request and routes
+        the request to the StatusController.
+
+        'statuses' is aliased here for backward compatibility with
+        neutron-lbaas LBaaS v2 API.
         """
         if id and len(remainder) and (remainder[0] == 'status' or
-                                      remainder[0] == 'stats'):
+                                      remainder[0] == 'statuses' or
+                                      remainder[0] == 'stats' or
+                                      remainder[0] == 'failover'):
             controller = remainder[0]
             remainder = remainder[1:]
-            if controller == 'status':
+            if controller == 'status' or controller == 'statuses':
                 return StatusController(lb_id=id), remainder
             elif controller == 'stats':
                 return StatisticsController(lb_id=id), remainder
+            elif controller == 'failover':
+                return FailoverController(lb_id=id), remainder
 
 
 class StatusController(base.BaseController):
@@ -528,3 +559,30 @@ class StatisticsController(base.BaseController, stats.StatsMixin):
         result = self._convert_db_to_type(
             lb_stats, lb_types.LoadBalancerStatisticsResponse)
         return lb_types.StatisticsRootResponse(stats=result)
+
+
+class FailoverController(LoadBalancersController):
+
+    def __init__(self, lb_id):
+        super(FailoverController, self).__init__()
+        self.lb_id = lb_id
+
+    @wsme_pecan.wsexpose(None, wtypes.text, status_code=202)
+    def put(self, **kwargs):
+        """Fails over a loadbalancer"""
+        context = pecan.request.context.get('octavia_context')
+        db_lb = self._get_db_lb(context.session, self.lb_id)
+
+        self._auth_validate_action(context, db_lb.project_id,
+                                   constants.RBAC_PUT_FAILOVER)
+
+        self._test_lb_status(context.session, self.lb_id)
+        try:
+            LOG.info("Sending failover request for lb %s to the handler",
+                     self.lb_id)
+            self.handler.failover(db_lb)
+        except Exception:
+            with excutils.save_and_reraise_exception(reraise=False):
+                self.repositories.load_balancer.update(
+                    context.session, self.lb_id,
+                    provisioning_status=constants.ERROR)
