@@ -17,6 +17,8 @@
 Routines for configuring Octavia
 """
 
+import sys
+
 from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
 from oslo_db import options as db_options
@@ -25,6 +27,7 @@ import oslo_messaging as messaging
 
 from octavia.common import constants
 from octavia.common import utils
+from octavia.i18n import _
 from octavia import version
 
 LOG = logging.getLogger(__name__)
@@ -140,6 +143,12 @@ networking_opts = [
                 help=_('List of network_ids that are valid for VIP '
                        'creation. If this field is empty, no validation '
                        'is performed.')),
+    cfg.ListOpt('reserved_ips',
+                default=['169.254.169.254'],
+                item_type=cfg.types.IPAddress(),
+                help=_('List of IP addresses reserved from being used for '
+                       'member addresses. IPv6 addresses should be in '
+                       'expanded, uppercase form.')),
 ]
 
 healthmanager_opts = [
@@ -200,6 +209,10 @@ oslo_messaging_opts = [
     cfg.StrOpt('event_stream_topic',
                default='neutron_lbaas_event',
                help=_('topic name for communicating events through a queue')),
+    cfg.StrOpt('event_stream_transport_url', default=None,
+               help=_('Transport URL to use for the neutron-lbaas'
+                      'synchronization event stream when neutron and octavia'
+                      'have separate queues.')),
 ]
 
 haproxy_amphora_opts = [
@@ -217,6 +230,13 @@ haproxy_amphora_opts = [
                default=5,
                help=_('Retry timeout between connection attempts in '
                       'seconds.')),
+    cfg.IntOpt('active_connection_max_retries',
+               default=15,
+               help=_('Retry threshold for connecting to active amphorae.')),
+    cfg.IntOpt('active_connection_rety_interval',
+               default=2,
+               help=_('Retry timeout between connection attempts in '
+                      'seconds for active amphora.')),
     cfg.IntOpt('build_rate_limit',
                default=-1,
                help=_('Number of amphorae that could be built per controller'
@@ -305,6 +325,11 @@ controller_worker_opts = [
                help=_('SSH key name used to boot the Amphora')),
     cfg.BoolOpt('amp_ssh_access_allowed',
                 default=True,
+                deprecated_for_removal=True,
+                deprecated_reason='This option and amp_ssh_key_name overlap '
+                                  'in functionality, and only one is needed. '
+                                  'SSH access can be enabled/disabled simply '
+                                  'by setting amp_ssh_key_name, or not.',
                 help=_('Determines whether or not to allow access '
                        'to the Amphorae')),
     cfg.ListOpt('amp_boot_network_list',
@@ -327,6 +352,9 @@ controller_worker_opts = [
     cfg.StrOpt('network_driver',
                default='network_noop_driver',
                help=_('Name of the network driver to use')),
+    cfg.StrOpt('distributor_driver',
+               default='distributor_noop_driver',
+               help=_('Name of the distributor driver to use')),
     cfg.StrOpt('loadbalancer_topology',
                default=constants.TOPOLOGY_SINGLE,
                choices=constants.SUPPORTED_LB_TOPOLOGIES,
@@ -585,6 +613,10 @@ def setup_logging(conf):
     product_name = "octavia"
     logging.setup(conf, product_name)
     LOG.info("Logging enabled!")
+    LOG.info("%(prog)s version %(version)s",
+             {'prog': sys.argv[0],
+              'version': version.version_info.release_string()})
+    LOG.debug("command line: %s", " ".join(sys.argv))
 
 
 # Use cfg.CONF.set_default to override the new configuration setting
