@@ -97,6 +97,7 @@ SUPPORTED_PROVISIONING_STATUSES = (ACTIVE, AMPHORA_ALLOCATED,
                                    PENDING_UPDATE, DELETED, ERROR)
 MUTABLE_STATUSES = (ACTIVE,)
 DELETABLE_STATUSES = (ACTIVE, ERROR)
+FAILOVERABLE_STATUSES = (ACTIVE, ERROR)
 
 SUPPORTED_AMPHORA_STATUSES = (AMPHORA_ALLOCATED, AMPHORA_BOOTING, ERROR,
                               AMPHORA_READY, DELETED,
@@ -177,6 +178,7 @@ FAILED_AMPHORA = 'failed_amphora'
 FAILOVER_AMPHORA = 'failover_amphora'
 AMPHORAE = 'amphorae'
 AMPHORA_ID = 'amphora_id'
+AMPHORA_INDEX = 'amphora_index'
 FAILOVER_AMPHORA_ID = 'failover_amphora_id'
 DELTA = 'delta'
 DELTAS = 'deltas'
@@ -210,6 +212,21 @@ ADDED_PORTS = 'added_ports'
 PORTS = 'ports'
 MEMBER_PORTS = 'member_ports'
 LOADBALANCER_TOPOLOGY = 'topology'
+HEALTH_MONITOR_ID = 'health_monitor_id'
+L7POLICY_ID = 'l7policy_id'
+L7RULE_ID = 'l7rule_id'
+LOAD_BALANCER_UPDATES = 'load_balancer_updates'
+LISTENER_UPDATES = 'listener_updates'
+POOL_UPDATES = 'pool_updates'
+MEMBER_UPDATES = 'member_updates'
+HEALTH_MONITOR_UPDATES = 'health_monitor_updates'
+L7POLICY_UPDATES = 'l7policy_updates'
+L7RULE_UPDATES = 'l7rule_updates'
+TIMEOUT_DICT = 'timeout_dict'
+REQ_CONN_TIMEOUT = 'req_conn_timeout'
+REQ_READ_TIMEOUT = 'req_read_timeout'
+CONN_MAX_RETRIES = 'conn_max_retries'
+CONN_RETRY_INTERVAL = 'conn_retry_interval'
 
 CERT_ROTATE_AMPHORA_FLOW = 'octavia-cert-rotate-amphora-flow'
 CREATE_AMPHORA_FLOW = 'octavia-create-amphora-flow'
@@ -243,6 +260,7 @@ UPDATE_MEMBER_FLOW = 'octavia-update-member-flow'
 UPDATE_POOL_FLOW = 'octavia-update-pool-flow'
 UPDATE_L7POLICY_FLOW = 'octavia-update-l7policy-flow'
 UPDATE_L7RULE_FLOW = 'octavia-update-l7rule-flow'
+UPDATE_AMPS_SUBFLOW = 'octavia-update-amps-subflow'
 
 POST_MAP_AMP_TO_LB_SUBFLOW = 'octavia-post-map-amp-to-lb-subflow'
 CREATE_AMP_FOR_LB_SUBFLOW = 'octavia-create-amp-for-lb-subflow'
@@ -276,6 +294,8 @@ AMP_VRRP_STOP = 'octavia-amphora-vrrp-stop'
 AMP_UPDATE_VRRP_INTF = 'octavia-amphora-update-vrrp-intf'
 CREATE_VRRP_GROUP_FOR_LB = 'octavia-create-vrrp-group-for-lb'
 CREATE_VRRP_SECURITY_RULES = 'octavia-create-vrrp-security-rules'
+AMP_COMPUTE_CONNECTIVITY_WAIT = 'octavia-amp-compute-connectivity-wait'
+AMP_LISTENER_UPDATE = 'octavia-amp-listeners-update'
 
 GENERATE_SERVER_PEM_TASK = 'GenerateServerPEMTask'
 
@@ -371,6 +391,11 @@ NO_CHECK = 'no check'
 # NO_CHECK = no health monitor is enabled
 HAPROXY_MEMBER_STATUSES = (UP, DOWN, DRAIN, MAINT, NO_CHECK)
 
+# Current maximum number of conccurent connections in HAProxy.
+# This is limited by the systemd "LimitNOFILE" and
+# the sysctl fs.file-max fs.nr_open settings in the image
+HAPROXY_MAX_MAXCONN = 1000000
+
 # Quota Constants
 QUOTA_UNLIMITED = -1
 MIN_QUOTA = QUOTA_UNLIMITED
@@ -378,6 +403,8 @@ MAX_QUOTA = 2000000000
 
 API_VERSION = '0.5'
 
+NOOP_EVENT_STREAMER = 'noop_event_streamer'
+
 HAPROXY_BASE_PEER_PORT = 1025
 KEEPALIVED_JINJA2_UPSTART = 'keepalived.upstart.j2'
 KEEPALIVED_JINJA2_SYSTEMD = 'keepalived.systemd.j2'
