@@ -674,16 +674,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         # if we run with anti-affinity we need to set the server group
         # as well
-        if CONF.nova.enable_anti_affinity:
-            lb = self._amphora_repo.get_all_lbs_on_amphora(
-                db_apis.get_session(), amp.id)
-            if lb:
-                stored_params[constants.SERVER_GROUP_ID] = (
-                    lb[0].server_group_id)
+        lb = self._amphora_repo.get_lb_for_amphora(
+            db_apis.get_session(), amp.id)
+        if CONF.nova.enable_anti_affinity and lb:
+            stored_params[constants.SERVER_GROUP_ID] = lb.server_group_id
 
         failover_amphora_tf = self._taskflow_load(
-            self._amphora_flows.get_failover_flow(role=amp.role,
-                                                  status=amp.status),
+            self._amphora_flows.get_failover_flow(
+                role=amp.role, load_balancer=lb),
             store=stored_params)
 
         with tf_logging.DynamicLoggingListener(
@@ -702,6 +700,10 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         try:
             amp = self._amphora_repo.get(db_apis.get_session(),
                                          id=amphora_id)
+            if not amp:
+                LOG.warning("Could not fetch Amphora %s from DB, ignoring "
+                            "failover request.", amphora_id)
+                return
             self._perform_amphora_failover(
                 amp, constants.LB_CREATE_FAILOVER_PRIORITY)
             if amp.load_balancer_id:
@@ -728,7 +730,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             lb = self._lb_repo.get(db_apis.get_session(),
                                    id=load_balancer_id)
 
-            amps = lb.amphorae
+            # Exclude amphora already deleted
+            amps = [a for a in lb.amphorae if a.status != constants.DELETED]
             for amp in amps:
                 # failover amphora in backup role
                 # Note: this amp may not currently be the backup
