@@ -17,7 +17,6 @@ import logging
 import time
 
 from oslo_config import cfg
-import six
 from stevedore import driver as stevedore_driver
 from taskflow import task
 from taskflow.types import failure
@@ -152,10 +151,8 @@ class DeleteAmphoraeOnLoadBalancer(BaseComputeTask):
     """
 
     def execute(self, loadbalancer):
-        for amp in six.moves.filter(
-            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
-                loadbalancer.amphorae):
-
+        for amp in loadbalancer.amphorae:
+            # The compute driver will already handle NotFound
             try:
                 self.compute.delete(amp.compute_id)
             except Exception:
@@ -176,7 +173,7 @@ class ComputeDelete(BaseComputeTask):
             raise
 
 
-class ComputeWait(BaseComputeTask):
+class ComputeActiveWait(BaseComputeTask):
     """Wait for the compute driver to mark the amphora active."""
 
     def execute(self, compute_id, amphora_id):
