@@ -448,9 +448,22 @@ class TestL7Policy(base.BaseAPITest):
         l7pos = self.get(self.L7POLICIES_PATH, params={
             'fields': ['id', 'project_id']}).json
         for l7po in l7pos['l7policies']:
-            self.assertIn(u'id', l7po.keys())
-            self.assertIn(u'project_id', l7po.keys())
-            self.assertNotIn(u'description', l7po.keys())
+            self.assertIn(u'id', l7po)
+            self.assertIn(u'project_id', l7po)
+            self.assertNotIn(u'description', l7po)
+
+    def test_get_one_fields_filter(self):
+        l7p1 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REJECT,
+            name='policy1').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        l7po = self.get(
+            self.L7POLICY_PATH.format(l7policy_id=l7p1.get('id')),
+            params={'fields': ['id', 'project_id']}).json.get(self.root_tag)
+        self.assertIn(u'id', l7po)
+        self.assertIn(u'project_id', l7po)
+        self.assertNotIn(u'description', l7po)
 
     def test_get_all_filter(self):
         policy1 = self.create_l7policy(
@@ -1006,3 +1019,13 @@ class TestL7Policy(base.BaseAPITest):
         self.delete(self.L7POLICY_PATH.format(
                     l7policy_id=l7policy.get('id')),
                     status=409)
+
+    def test_delete_already_deleted(self):
+        l7policy = self.create_l7policy(self.listener_id,
+                                        constants.L7POLICY_ACTION_REJECT,
+                                        ).get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(self.lb_id, status=constants.DELETED)
+        self.delete(self.L7POLICY_PATH.format(
+                    l7policy_id=l7policy.get('id')),
+                    status=204)
