@@ -269,6 +269,17 @@ class TestValidations(base.TestCase):
             net_mock.return_value.get_port.return_value = port
             self.assertEqual(validate.port_exists(port_id), port)
 
+    def test_check_port_in_use(self):
+        port_id = uuidutils.generate_uuid()
+        device_id = uuidutils.generate_uuid()
+        port = network_models.Port(id=port_id, device_id=device_id)
+        with mock.patch(
+                 'octavia.common.utils.get_network_driver') as net_mock:
+            net_mock.return_value.get_port.device_id = port
+            self.assertRaises(
+                exceptions.ValidationException,
+                validate.check_port_in_use, port)
+
     def test_subnet_exists_with_bad_subnet(self):
         subnet_id = uuidutils.generate_uuid()
         with mock.patch(
@@ -347,3 +358,87 @@ class TestValidations(base.TestCase):
         self.assertRaises(
             exceptions.ValidationException,
             validate.network_allowed_by_config, net_id3)
+
+    def test_qos_policy_exists(self):
+        qos_policy_id = uuidutils.generate_uuid()
+        qos_policy = network_models.QosPolicy(id=qos_policy_id)
+        with mock.patch(
+                'octavia.common.utils.get_network_driver') as net_mock:
+            net_mock.return_value.get_qos_policy.return_value = qos_policy
+            self.assertEqual(
+                validate.qos_policy_exists(qos_policy_id),
+                qos_policy)
+
+            net_mock.return_value.get_qos_policy.side_effect = Exception
+            self.assertRaises(exceptions.InvalidSubresource,
+                              validate.qos_policy_exists,
+                              qos_policy_id)
+
+    def test_check_session_persistence(self):
+        valid_cookie_name_dict = {'type': 'APP_COOKIE',
+                                  'cookie_name': 'chocolate_chip'}
+        invalid_cookie_name_dict = {'type': 'APP_COOKIE',
+                                    'cookie_name': '@chocolate_chip'}
+        invalid_type_HTTP_cookie_name_dict = {'type': 'HTTP_COOKIE',
+                                              'cookie_name': 'chocolate_chip'}
+        invalid_type_IP_cookie_name_dict = {'type': 'SOURCE_IP',
+                                            'cookie_name': 'chocolate_chip'}
+        invalid_missing_cookie_name_dict = {'type': 'APP_COOKIE'}
+
+        # Validate that a good cookie name passes
+        validate.check_session_persistence(valid_cookie_name_dict)
+
+        # Test raises with providing an invalid cookie name
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_session_persistence,
+                          invalid_cookie_name_dict)
+
+        # Test raises type HTTP_COOKIE and providing cookie_name
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_session_persistence,
+                          invalid_type_HTTP_cookie_name_dict)
+
+        # Test raises type SOURCE_IP and providing cookie_name
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_session_persistence,
+                          invalid_type_IP_cookie_name_dict)
+
+        # Test raises when type APP_COOKIE but no cookie_name
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_session_persistence,
+                          invalid_missing_cookie_name_dict)
+
+        # Test catch all exception raises a user friendly message
+        with mock.patch('re.compile') as compile_mock:
+            compile_mock.side_effect = Exception
+            self.assertRaises(exceptions.ValidationException,
+                              validate.check_session_persistence,
+                              valid_cookie_name_dict)
+
+    def test_ip_not_reserved(self):
+        self.conf.config(group="networking", reserved_ips=['198.51.100.4'])
+
+        # Test good address
+        validate.ip_not_reserved('203.0.113.5')
+
+        # Test IPv4 reserved address
+        self.assertRaises(exceptions.InvalidOption,
+                          validate.ip_not_reserved,
+                          '198.51.100.4')
+
+        self.conf.config(
+            group="networking",
+            reserved_ips=['2001:0DB8:0000:0000:0000:0000:0000:0005'])
+
+        # Test good IPv6 address
+        validate.ip_not_reserved('2001:0DB8::9')
+
+        # Test reserved IPv6 expanded
+        self.assertRaises(exceptions.InvalidOption,
+                          validate.ip_not_reserved,
+                          '2001:0DB8:0000:0000:0000:0000:0000:0005')
+
+        # Test reserved IPv6 short hand notation
+        self.assertRaises(exceptions.InvalidOption,
+                          validate.ip_not_reserved,
+                          '2001:0DB8::5')
