@@ -14,9 +14,12 @@
 #
 
 import mock
+from oslo_config import cfg
+from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 from taskflow.types import failure
 
+from octavia.amphorae.driver_exceptions import exceptions as driver_except
 from octavia.common import constants
 from octavia.common import data_models
 from octavia.controller.worker.tasks import amphora_driver_tasks
@@ -28,6 +31,8 @@ AMP_ID = uuidutils.generate_uuid()
 COMPUTE_ID = uuidutils.generate_uuid()
 LISTENER_ID = uuidutils.generate_uuid()
 LB_ID = uuidutils.generate_uuid()
+CONN_MAX_RETRIES = 10
+CONN_RETRY_INTERVAL = 6
 
 _amphora_mock = mock.MagicMock()
 _amphora_mock.id = AMP_ID
@@ -61,8 +66,42 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         _LB_mock.amphorae = [_amphora_mock]
         _LB_mock.id = LB_ID
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora",
+                    active_connection_max_retries=CONN_MAX_RETRIES)
+        conf.config(group="haproxy_amphora",
+                    active_connection_rety_interval=CONN_RETRY_INTERVAL)
         super(TestAmphoraDriverTasks, self).setUp()
 
+    def test_amp_listener_update(self,
+                                 mock_driver,
+                                 mock_generate_uuid,
+                                 mock_log,
+                                 mock_get_session,
+                                 mock_listener_repo_get,
+                                 mock_listener_repo_update,
+                                 mock_amphora_repo_update):
+
+        timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
+                        constants.REQ_READ_TIMEOUT: 2,
+                        constants.CONN_MAX_RETRIES: 3,
+                        constants.CONN_RETRY_INTERVAL: 4}
+
+        amp_list_update_obj = amphora_driver_tasks.AmpListenersUpdate()
+        amp_list_update_obj.execute([_listener_mock], 0,
+                                    [_amphora_mock], timeout_dict)
+
+        mock_driver.update_amphora_listeners.assert_called_once_with(
+            [_listener_mock], 0, [_amphora_mock], timeout_dict)
+
+        mock_driver.update_amphora_listeners.side_effect = Exception('boom')
+
+        amp_list_update_obj.execute([_listener_mock], 0,
+                                    [_amphora_mock], timeout_dict)
+
+        mock_amphora_repo_update.assert_called_once_with(
+            _session_mock, AMP_ID, status=constants.ERROR)
+
     def test_listener_update(self,
                              mock_driver,
                              mock_generate_uuid,
@@ -480,10 +519,15 @@ class TestAmphoraDriverTasks(base.TestCase):
                                            mock_listener_repo_update,
                                            mock_amphora_repo_update):
         _LB_mock.amphorae = _amphorae_mock
+
+        timeout_dict = {constants.CONN_MAX_RETRIES: CONN_MAX_RETRIES,
+                        constants.CONN_RETRY_INTERVAL: CONN_RETRY_INTERVAL}
+
         amphora_update_vrrp_interface_obj = (
             amphora_driver_tasks.AmphoraUpdateVRRPInterface())
         amphora_update_vrrp_interface_obj.execute(_LB_mock)
-        mock_driver.get_vrrp_interface.assert_called_once_with(_amphora_mock)
+        mock_driver.get_vrrp_interface.assert_called_once_with(
+            _amphora_mock, timeout_dict=timeout_dict)
 
         # Test revert
         mock_driver.reset_mock()
@@ -550,3 +594,22 @@ class TestAmphoraDriverTasks(base.TestCase):
             amphora_driver_tasks.AmphoraVRRPStart())
         amphora_vrrp_start_obj.execute(_LB_mock)
         mock_driver.start_vrrp_service.assert_called_once_with(_LB_mock)
+
+    def test_amphora_compute_connectivity_wait(self,
+                                               mock_driver,
+                                               mock_generate_uuid,
+                                               mock_log,
+                                               mock_get_session,
+                                               mock_listener_repo_get,
+                                               mock_listener_repo_update,
+                                               mock_amphora_repo_update):
+        amp_compute_conn_wait_obj = (
+            amphora_driver_tasks.AmphoraComputeConnectivityWait())
+        amp_compute_conn_wait_obj.execute(_amphora_mock)
+        mock_driver.get_info.assert_called_once_with(_amphora_mock)
+
+        mock_driver.get_info.side_effect = driver_except.TimeOutException()
+        self.assertRaises(driver_except.TimeOutException,
+                          amp_compute_conn_wait_obj.execute, _amphora_mock)
+        mock_amphora_repo_update.assert_called_once_with(
+            _session_mock, AMP_ID, status=constants.ERROR)
